package debug;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import org.testng.annotations.AfterClass;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

public class ParallelTest {

    final Map<Long, Long> tracker = new ConcurrentHashMap<>();

    @DataProvider(parallel = true, name = "dataProvider")
    public Iterator<Object[]> dataProvider() {
        List<Object[]> data = new ArrayList<>();

        for (int i = 0; i < 100000; i++) {
            data.add(new Object[]{"ID_" + i});
        }
        return data.iterator();
    }

    @Test(dataProvider = "dataProvider", threadPoolSize = 50, invocationCount = 10)
    public void testParallelDataProvider(String iterId) {
        Long threadId = Thread.currentThread().getId();
        System.out.println(String.format("[tId:%d] ParallelTest.testParallelDataProvider(%s)", threadId, iterId));
        Long count = tracker.get(threadId);
        if (count == null) {
            tracker.put(threadId, (long) 1);
        } else {
            tracker.put(threadId, count + 1);
        }
    }

    @AfterClass
    public void stats() {
        long total = 0l;
        
        System.out.println(String.format("Total threads: %d", tracker.size()));
        for (long tId : tracker.keySet()) {
            long count = tracker.get(tId);
            System.out.println(String.format("  Thread[%d] : %d iterations", tId, count));
            total += count;
        }
        System.out.println(String.format("Total iterations: %d", total));
    }
}